@@ -1,6 +1,6 @@
 from . import private
 
-def load_linelist(band, l_type):
+def load_linelist(band, table):
 
     '''
     Load the YJ-band LDR line list and LDR-Teff relation coefficients.
@@ -9,8 +9,9 @@ def load_linelist(band, l_type):
     ----------
     band : string
         Specify the band of relation set to load. Have to be one of the following: "h" or "yj".
-    l_type : string
-        Specify the type of relation set to load. If band is "h", then have to be "giant"; if band is "yj", then have to be one of the following: "dwarf-j20a", "giant-t18" or "supergiant-j20a"
+    table : string
+        Specify the table of relation set to load. If band is "h", then have to be "giant"; if band is "yj", then have to be one of the following: "dwarf-j20a", "giant-t18", "supergiant-j20a", "dwarf-j21" or "giant-j21". 
+        You can also specify a path to a custom line list (must be a .csv file) following the same format as dwarf-j20a; in this case the variable band will not be used. 
 
     Returns
     ----------
@@ -18,19 +19,23 @@ def load_linelist(band, l_type):
         DataFrame containing LDR linelist and LDR-Teff relations.
 
     '''
-    l_type_dict = {'dwarf-j20a':'dwarf_j20a', 'giant-t18':'giant_t18', 'supergiant-j20a':'spg_j20a'}
+    if table[-4:] == '.csv':
+        df = private.pd.read_csv(table)
+        return df
+
+    l_type_dict = {'dwarf-j20a':'dwarf_j20a', 'giant-t18':'giant_t18', 'supergiant-j20a':'spg_j20a', 'dwarf-j21':'dwarf_j21', 'giant-j21':'giant_j21'}
     band = band.lower()
 
     if band == 'h':
-        if l_type == 'giant':
+        if table == 'giant':
             df = private.pd.read_csv(__path__[0] + '/file/h-ldr/lineratio_giant.csv')
         else:
-            raise ValueError('Band or l_type incorrect.')
+            raise ValueError('Band or table incorrect.')
     elif band == 'yj':
-        if l_type in ['dwarf-j20a', 'giant-t18', 'supergiant-j20a']:
-            df = private.pd.read_csv(__path__[0] + '/file/yj-ldr/lineratio_all_{}.csv'.format(l_type_dict[l_type]))
+        if table in l_type_dict.keys():
+            df = private.pd.read_csv(__path__[0] + '/file/yj-ldr/lineratio_all_{}.csv'.format(l_type_dict[table]))
         else:
-            raise ValueError('Band or l_type incorrect.')
+            raise ValueError('Band or table incorrect.')
     return df
 
 def lineele2ele(lineelement):
@@ -102,7 +107,7 @@ def depth_measure(wav, flux, line_input, suffix=False, S_N=False, func='parabola
         Linelist to be measured.
 
     suffix : int or str, optional
-        Suffix of columns of the output pandas DataFrame. 1 for low EP line and 2 for high EP line. If set to False, no suffix will be added, but it cannot be used to calculate the LDR in cal_LDR.
+        Suffix of columns of the output pandas DataFrame. 1 for low EP line and 2 for high EP line. If set to False, no suffix will be added, but it cannot be used to calculate the LDR in cal_ldr.
 
     S_N : int, float or list with length 2, optional
         Signal to noise ratio of the spectrum.
@@ -130,14 +135,14 @@ def depth_measure(wav, flux, line_input, suffix=False, S_N=False, func='parabola
     depth_measure_list = []
 
     # Calculate the error of depth if S_N is provided.
-    if type(S_N) in [int, float]:
+    if type(S_N) in [int, float, private.np.float, private.np.float32, private.np.float64]:
         d_err = 1 / S_N
     elif type(S_N) == list and len(S_N) == 2:
         d_err = (1/S_N[0]**2 + 1/S_N[1]**2)**0.5
     elif S_N == False:
         pass
     else:
-        raise TypeError('The type or length of S_N is incorrect.')
+        raise TypeError('The type or length of S_N is incorrect. Please convert it to one of the following: int, float, private.np.float, private.np.float32, private.np.float64')
 
     # Do a loop for all the lines inside linelist.
     for line in line_input:
@@ -179,6 +184,7 @@ def depth_measure(wav, flux, line_input, suffix=False, S_N=False, func='parabola
                 poly_err = d_err
             poly_flag = 0
             if poly_res[0][0] < 0 or poly_depth < 0:
+                poly_depth, poly_del_wav, poly_err = private.np.nan, private.np.nan, private.np.nan
                 poly_flag = 2
             elif abs(poly_del_wav) > del_wav_lim:
                 poly_flag = 1
@@ -193,6 +199,7 @@ def depth_measure(wav, flux, line_input, suffix=False, S_N=False, func='parabola
             try:
                 gauss_res = private.curve_fit(private.Gauss_func, sub_wav, sub_flux, p0=[0.5, line, 1])
             except:
+                gauss_depth, gauss_del_wav, gauss_err = private.np.nan, private.np.nan, private.np.nan
                 gauss_flag = 2
             else:
                 gauss_depth = gauss_res[0][0]
@@ -216,6 +223,7 @@ def depth_measure(wav, flux, line_input, suffix=False, S_N=False, func='parabola
                 GH_res = private.curve_fit(private.GH_func, sub_wav, sub_flux, p0=[private.np.min(sub_flux), line, 0.8, 0, 0])
                 GH_res = GH_res[0]
             except:
+                GH_depth, GH_del_wav, GH_err = private.np.nan, private.np.nan, private.np.nan
                 GH_flag = 2
             else:
                 # x = private.np.arange(private.np.min(sub_wav), private.np.max(sub_wav), 0.01)
@@ -263,15 +271,15 @@ def cal_ldr(depth_pd_1, depth_pd_2, type='LDR', flag=True):
         The depth measurement (output) of depth_measure with suffix 2. They act as dividends in LDR.
 
     type : str, default 'LDR'
-        The type of LDR to be calculated. Have to be 'ldr' or 'lgldr' (log10).
+        The type of LDR to be calculated. Have to be 'ldr' or 'logldr' (log10).
 
     flag : bool, default True
-        When True the lgLDR and error values of the line pairs with flag1 or flag2 not equal 0 will be set as NaN.
+        When True the logLDR and error values of the line pairs with flag1 or flag2 not equal 0 will be set as NaN.
 
     Return
     ----------
     LDR_pd : pandas.DataFrame
-        A DataFrame containing the LDR/lgLDR values and their errors.
+        A DataFrame containing the LDR/logLDR values and their errors.
     '''
 
     d1 = depth_pd_1['depth1'].values
@@ -285,14 +293,15 @@ def cal_ldr(depth_pd_1, depth_pd_2, type='LDR', flag=True):
     if type.lower() == 'ldr':
         LDR_pd = private.np.column_stack([LDR, err_LDR])
         LDR_pd = private.pd.DataFrame(LDR_pd, columns=['LDR', 'LDR_error'])
-    elif type.lower() == 'lgldr':
+    elif type.lower() == 'logldr':
         LDR[LDR < 0] = private.np.nan
-        lgLDR = private.np.log10(LDR)
-        err_lgLDR = 1/(LDR*private.np.log(10))*err_LDR
-        LDR_pd = private.np.column_stack([lgLDR, err_lgLDR])
-        LDR_pd = private.pd.DataFrame(LDR_pd, columns=['lgLDR', 'lgLDR_error'])
+        logLDR = private.np.log10(LDR)
+        err_logLDR = 1/(LDR*private.np.log(10))*err_LDR
+        # err_logLDR = 1 / private.np.log(10) * private.np.sqrt(d1_err**2 / d1**2 + d2_err**2 / d2**2)
+        LDR_pd = private.np.column_stack([logLDR, err_logLDR])
+        LDR_pd = private.pd.DataFrame(LDR_pd, columns=['logLDR', 'logLDR_error'])
     if flag:
-        LDR_pd.at[((depth_pd_1['flag1'] != 0)|(depth_pd_2['flag2'] != 0)).values,:] = private.np.nan
+        LDR_pd.at[(depth_pd_1['flag1'] != 0).values | (depth_pd_2['flag2'] != 0).values,:] = private.np.nan
     return LDR_pd
 
 def combine_df(df_list, remove_line_wav=True):
@@ -312,8 +321,10 @@ def combine_df(df_list, remove_line_wav=True):
     output_df : pandas.DataFrame
         A DataFrame containing the combined values.
     '''
-    for i in range(len(df_list)):
-        if df_list[i].index[0] != 0:
+    for i in range(4):
+        if i not in [1,2]:
+            df_list[i] = df_list[i].reset_index(drop=True)
+        else:
             df_list[i] = df_list[i].reset_index()
 
     output_df = private.pd.concat(df_list, axis=1)
@@ -432,19 +443,18 @@ def ldr2tldr_winered_solar(df, sigma_clip=0, df_output=False):
         The DataFrame containing T_LDRi.
     '''
 
-    # Calculate the T_LDR and _TLDR_error. For T18 line set only std_res is
-    # used, while for others the confidence interval is calculated.
-    df['T_LDRi'] = (df['lgLDR']) * df['slope'] + df['intercept']
-    T_err_r = df['lgLDR_error'] * df['slope']
+    # Calculate the T_LDR and _TLDR_error.
+    df['T_LDRi'] = (df['logLDR']) * df['slope'] + df['intercept']
+    T_err_r = df['logLDR_error'] * df['slope']
     try:
-        T_err_fit = private.t.ppf(1-0.32/2, df['Npoints']-2) * df['std_res'] * (1 + 1 / df['Npoints'] + (df['lgLDR']-df['mean_lgLDR'])**2 / (df['Npoints']-1) / df['std_lgLDR']**2)**0.5
+        T_err_fit = private.t.ppf(1-0.32/2, df['Npoints']-2) * df['std_res'] * (1 + 1 / df['Npoints'] + (df['logLDR']-df['mean_logLDR'])**2 / (df['Npoints']-1) / df['std_logLDR']**2)**0.5
         df['T_LDRi_error'] = (T_err_r**2 + T_err_fit**2)**0.5
     except KeyError:
         df['T_LDRi_error'] = (T_err_r**2 + df['std_res']**2)**0.5
 
-    # Exclude those outside the lgLDR range.
+    # Exclude those outside the logLDR range.
     try:
-        pointer = (df['lgLDR'] > df['max_lgLDR']) | (df['lgLDR'] < df['min_lgLDR'])
+        pointer = (df['logLDR'] > df['max_logLDR']) | (df['logLDR'] < df['min_logLDR'])
         df.at[pointer, 'T_LDRi'] = private.np.nan
         df.at[pointer, 'T_LDRi_error'] = private.np.nan
     except KeyError:
@@ -469,7 +479,7 @@ def ldr2tldr_winered_solar(df, sigma_clip=0, df_output=False):
             weights = 1/df[pointer]['T_LDRi_error']**2
             T_LDR = private.np.average(df[pointer]['T_LDRi'], weights=weights)
         elif sigma_clip < 0:
-            raise ValueError('Sigma_clip have to be >= 0.')
+            raise ValueError('sigma_clip have to be >= 0.')
 
         T_LDR_err = 1 / private.np.sum(1 / df[pointer]['T_LDRi_error']**2)**0.5
 
@@ -477,3 +487,123 @@ def ldr2tldr_winered_solar(df, sigma_clip=0, df_output=False):
         return T_LDR, T_LDR_err, df
     else:
         return T_LDR, T_LDR_err
+
+def poly2_mmtf(b, x):
+    '''
+    x[1]: Teff, x[1]: [Fe/H]
+    log(r) = b[0] + b[1]*T + b[2]*T**2 + b[3]*M + b[4]*M*T 
+    '''
+    b_nonan = [0 if private.np.isnan(i) else i for i in b]
+    return b_nonan[0] + b_nonan[1]*x[0] + b_nonan[2]*x[0]**2 + b_nonan[3]*x[1] + b_nonan[4]*x[0]*x[1]
+
+def cal_posterior(record_all, l_type, plot=False, likelihood_out=False):
+    N = 0
+    for index in range(len(record_all)):
+        
+        # Skip if log_LDR_err > 0.1 or no measurement
+        if record_all.loc[index, 'log_LDR_err'] > 0.1 or private.np.isnan(record_all.loc[index, 'log_LDR']):
+            continue
+        else:
+            log_LDR = record_all.loc[index, 'log_LDR']
+            log_LDR_err = record_all.loc[index, 'log_LDR_err']
+            
+        if N == 0:
+            index_init = index
+        if l_type == 'dwarf':
+            X = private.np.arange(4400, 6700, 1) / 1000
+            Y = private.np.arange(-0.9, 0.6, 0.005)
+        elif l_type == 'giant':
+            X = private.np.arange(3000, 5500, 1) / 1000
+            Y = private.np.arange(-0.9, 0.4, 0.005)
+        else:
+            raise ValueError("l_type can only be 'dwarf' or 'giant'!")
+        X, Y = private.np.meshgrid(X, Y)
+        Z = poly2_mmtf(record_all.loc[index, ['a', 'b', 'c', 'd', 'e']].values, [X, Y])
+
+        if record_all.loc[index, 'type'] in ['A', 'B']:
+            # For each X
+            pre_interval = private.derive_fitting_interval(*list(record_all.loc[index, ['N', 'sigma', 'mean_T', 'std_T']].values), 1, X)
+        else:
+            # For each X and Y
+            pre_interval = private.derive_fitting_interval_2d(*list(record_all.loc[index, ['N', 'sigma', 'mean_T', 'std_T', 'mean_FeH', 'std_FeH']].values),
+                                                                      2, X, Y)
+        sigma = private.np.sqrt(pre_interval**2 + record_all.loc[index, 'sigma_r']**2 + log_LDR_err**2)
+
+        log_likelihood_i = - (0.5*private.np.log10(2 * private.np.pi * sigma ** 2) + (Z - log_LDR) ** 2 / sigma**2 / 2 * private.np.log10(private.np.e))
+        #  print(log_likelihood_i)
+        N = N + 1
+        if index == index_init:
+                log_likelihood = log_likelihood_i
+        else:
+            log_likelihood = log_likelihood_i + log_likelihood
+    
+    # Return np.nans if no measurement is available
+    if N == 0:
+        return [private.np.nan, private.np.nan, private.np.nan, private.np.nan, private.np.nan, private.np.nan]
+    else:
+        log_likelihood = log_likelihood - private.np.max(log_likelihood)
+        
+    likelihood = 10**log_likelihood
+    likelihood_x = private.np.sum(likelihood, axis=0) / private.np.sum(private.np.sum(likelihood, axis=0))
+    likelihood_y = private.np.sum(likelihood, axis=1) / private.np.sum(private.np.sum(likelihood, axis=1))
+    max_index = private.np.unravel_index(likelihood.argmax(), likelihood.shape)
+    Tldr = X[0][max_index[1]]
+    Mldr = Y[:, 0][max_index[0]]
+
+    Tldr_low_err, Tldr_high_err = private.credi_interval(likelihood_x, X[0], Tldr)
+    Tldr = Tldr * 1000
+    Tldr_low_err, Tldr_high_err = Tldr_low_err*1000, Tldr_high_err*1000
+
+    Mldr_low_err, Mldr_high_err = private.credi_interval(likelihood_y, Y[:, 0], Mldr)
+    
+    # Plot
+    if plot:
+        left, width = 0.17, 0.65
+        bottom, height = 0.14, 0.65
+        spacing = 0.005
+
+        rect_scatter = [left, bottom, width, height]
+        rect_histx = [left, bottom + height + spacing+0.0015, width, 0.2]
+        rect_histy = [left + width + spacing, bottom, 0.17, height]
+
+        ax_contour = private.plt.axes(rect_scatter)
+        ax_contour.tick_params(direction='in', top=True, right=True)
+        ax_px = private.plt.axes(rect_histx)
+        ax_px.tick_params(direction='in', labelbottom=False, labelleft=False)
+        ax_py = private.plt.axes(rect_histy)
+        ax_py.tick_params(direction='in', labelbottom=False, labelleft=False)
+
+        CS = ax_contour.contour(X*1000, Y, likelihood, [1-0.997, 1-0.95, 1-0.68], 
+                     colors='k')
+        # CS.levels = ['$3\sigma$', '$2\sigma$', '$1\sigma$']
+        # ax_contour.clabel(CS, fontsize=10, inline_spacing=2)
+
+        ax_contour.set_xlim(Tldr-200, Tldr+200)
+        ax_contour.set_ylim(Mldr-0.2, Mldr+0.2)
+
+        ax_px.set_xlim(ax_contour.get_xlim())
+        ax_py.set_ylim(ax_contour.get_ylim())
+
+        ax_contour.set_xlabel('$T_\mathrm{eff}$ (K)')
+        ax_contour.set_ylabel('[Fe/H]')
+
+        ax_contour.scatter(Tldr, Mldr)
+        ax_contour.axvline(Tldr_low_err+Tldr, c='brown', alpha=0.5, linestyle='--')
+        ax_contour.axvline(Tldr_high_err+Tldr, c='brown', alpha=0.5, linestyle='--')
+        ax_contour.axhline(Mldr_low_err+Mldr, c='brown', alpha=0.5, linestyle='--')
+        ax_contour.axhline(Mldr_high_err+Mldr, c='brown', alpha=0.5, linestyle='--')
+        ax_px.axvline(Tldr_low_err+Tldr, c='brown', alpha=0.5, linestyle='--')
+        ax_px.axvline(Tldr_high_err+Tldr, c='brown', alpha=0.5, linestyle='--')
+        ax_py.axhline(Mldr_low_err+Mldr, c='brown', alpha=0.5, linestyle='--')
+        ax_py.axhline(Mldr_high_err+Mldr, c='brown', alpha=0.5, linestyle='--')
+
+        # Plot the P
+        ax_px.plot(X[0]*1000, likelihood_x, c='k')
+        ax_py.plot(likelihood_y, Y[:, 0], c='k')
+        ax_contour.text(0.05, 0.65+0.125, '$T_\mathrm{{LDR}}={0:.0f}^{{ +{2:.0f} }}_{{ {1:.0f} }}$\n$M_\mathrm{{LDR}}$=${3:.2f}^{{ +{5:.2f} }}_{{ {4:.2f} }}$'.format(Tldr, Tldr_low_err, Tldr_high_err, 
+                                                                                           Mldr, Mldr_low_err, Mldr_high_err),
+                        transform=ax_contour.transAxes, fontsize=12)
+    if likelihood_out:
+        return [Tldr, Tldr_low_err, Tldr_high_err, Mldr, Mldr_low_err, Mldr_high_err], likelihood
+    else:
+        return [Tldr, Tldr_low_err, Tldr_high_err, Mldr, Mldr_low_err, Mldr_high_err]
\ No newline at end of file
